<?php
/**
* Plugin Plinstagram
*
* @package	PLX
* @version	1.0
* @date	14/02/22
* @author 
**/
class Plinstagram extends plxPlugin {
		private $plinstagram_page_name;
		private $linkname;
		private $filename_base='Plinstagram_picture_';
		private $max_size=1200;
		private $plxMotor,$plxAdmin,$plxPlugins,$plxShow;
		private $catId;
		private $medias_path=PLX_ROOT.'data/medias/_PLINSTAGRAM';

	public function __construct($default_lang) {
		# appel du constructeur de la classe plxPlugin (obligatoire)
		parent::__construct($default_lang);
		
		# limite l'acces a l'ecran d'administration du plugin
		# PROFIL_ADMIN , PROFIL_MANAGER , PROFIL_MODERATOR , PROFIL_EDITOR , PROFIL_WRITER
		$this->setAdminProfil(PROFIL_ADMIN);		
		
		# limite l'acces a l'ecran de configuration du plugin
		# PROFIL_ADMIN , PROFIL_MANAGER , PROFIL_MODERATOR , PROFIL_EDITOR , PROFIL_WRITER
		$this->setConfigProfil(PROFIL_ADMIN);		
		
		$this->max_size=$this->getParam("max_size");
		$this->linkname=$this->getParam("linkname");
		$this->plinstagram_page_name=$this->getParam("plinstagram_page_name");



		# Declaration d'un hook (existant ou nouveau)
		$this->addHook('IndexBegin','IndexBegin');
		$this->addHook('AdminIndexPrepend','AdminIndexPrepend');
		$this->addHook('AdminArticlePrepend','AdminArticlePrepend');
		$this->addHook('ThemeEndBody','ThemeEndBody');
		$this->addHook('ThemeEndHead','ThemeEndHead');
		$this->addHook('plxAdminEditArticle','plxAdminEditArticle');
		$this->addHook('plxAdminEditArticleXml','plxAdminEditArticleXml');
		$this->addHook('AdminProfilTop','AdminProfilTop');
		$this->addHook('showBookmarkletLink','showBookmarkletLink');
		$this->addHook('getTagList','getTagList');
		$this->addHook('showLinksPageLink','showLinksPageLink');
		$this->addHook('showPinstagramRSSLink','showPinstagramRSSLink');
		//$this->addHook('plxFeedRssArticlesXml','plxFeedRssArticlesXml');
		$this->addHook('catId2Name','catId2Name'); // utile pour style spécifiquement les articles ayant la classe .plinstagram

	}

	# Activation / desactivation
	public function OnActivate() {
		# creates default config values 
		if (empty($this->plinstagram_page_name)){
			$this->plinstagram_page_name='Plinstagram';
			$this->setParam("plinstagram_page_name", $this->plinstagram_page_name, "string");
		}		
		if (empty($this->linkname)){
			$this->linkname='Poster sur Plinstagram';
			$this->setParam("linkname", $this->linkname, "string");
		}
		
		if (empty($this->max_size)){
			$this->max_size=1200;
			$this->setParam("max_size", $this->max_size, "numeric");
		}

		# creates the Plinstagram category if not present
		$cats=$this->plxAdmin->aCats;
		$cats2=array();
		$max=0;
		foreach ($cats as $id => $data) {
			$cats2[$data['name']]=$data;
			$max=max($max,intval($id));
		}
		
		if (empty($cats2['Plinstagram'])){
			$max++;
			$max="$max";
			$max=str_repeat('0', 3-strlen($max)).$max;
			$newcat='<categorie number="'.$max.'" active="1" tri="desc" bypage="5" menu="non" url="'.plxUtils::title2url('Plinstagram').'" template="categorie.php"><name><![CDATA[Plinstagram]]></name><description><![CDATA[Plinstagram page]]></description><meta_description><![CDATA[Plinstagram page]]></meta_description><meta_keywords><![CDATA[Plinstagram]]></meta_keywords><title_htmltag><![CDATA[Plinstagram]]></title_htmltag></categorie>';
			$categories=file_get_contents('../../'.PLX_CONFIG_PATH.'categories.xml');
			$categories=str_replace('</document>',$newcat.'</document>',$categories);
			file_put_contents('../../'.PLX_CONFIG_PATH.'categories.xml', $categories);
		}

		// plinstagram destination folder (for sent pics)
		if (!is_dir($this->medias_path)){mkdir($this->medias_path);}
	}
	public function OnDeactivate() {
		# code à executer à la désactivation du plugin
	}
	

	########################################
	# HOOKS
	########################################
	########################################
	# ThemeEndBody
	########################################
	# Description: ajoute le js de la galerie à la page
	public function ThemeEndBody(){
		echo '
		<script type="text/javascript" src="'.PLX_PLUGINS.'Plinstagram/lightbox.js"></script>';
	}


	########################################
	# ThemeEndHead
	########################################
	# Description: ajoute le css de plinstagram à la page
	public function ThemeEndHead(){
		echo '<link rel="stylesheet" type="text/css" href="'.PLX_PLUGINS.'Plinstagram/style.css"/>';
	}
########################################
	# AdminIndexPrepend
	########################################
	# Description: redirige vers le formulaire de plinstagram 
	# après le login si on n'était pas connecté
	public function AdminIndexPrepend(){
		if (!empty($_SESSION['redirect'])){
			$redirect=$_SESSION['redirect'];
			unset($_SESSION['redirect']);
			$this->redirect($redirect);
		}
	}
	########################################
	# IndexBegin
	########################################
	# Description: ici, on crée le formulaire de plinstagram si demandé
	public function IndexBegin(){
		if (!is_dir($this->medias_path)){mkdir($this->medias_path);}
		$this->setPlxVars();
		$this->setCatId();

		if (!empty($_GET['source'])&&$_GET['source']=='plinstagram'){
			if (empty($_SESSION['user'])){ 
				# Si on n'est pas connecté on redirige vers le login d'abord
				# en préparant l'adresse de retour via SESSION (utilisée ensuite dans le hook)
				# AdminIndexPrepend()
				$_SESSION['redirect']=plxUtils::getRacine().'?'.$_SERVER['QUERY_STRING'];
				$this->redirect(plxUtils::getRacine().'/core/admin/auth.php');# ça c'est moche mais ça marche ¯\_(ツ)_/¯
			}

			# gestion de la génération du formulaire si on vient du bookmarklet
			$this->max_size				   = empty($this->max_size)?1200:$this->max_size;
			$user						   = preg_replace('#[^0-9]#','',$_GET['user']);
			$artId                         = $this->newIdArticle();	
			$login                         = $this->plxMotor->aUsers[strip_tags($user)]['login'];
			//$from                          =!empty($_GET['from'])?strip_tags($_GET['from']):'';
			$media_link					   ="
			var title = 'medias';window.open('".plxUtils::getRacine()."admin/medias.php','','menubar=no,height=' + screen.height +',width=' + screen.width+',toolbar=no,scrollbars=auto,status=no,top=0,left=0');
			";
			

			echo '<html><head><title>Publier sur Pinstagram</title><meta http-equiv="Content-Type" content="text/html; charset=utf-8" />';
			echo '
			
			<link rel="stylesheet" href="./'.$this->plxShow->plxMotor->aConf['racine_themes'].$this->plxShow->plxMotor->style.'/css/theme.css"/>
			<link rel="stylesheet" href="./'.$this->plxShow->plxMotor->aConf['racine_themes'].$this->plxShow->plxMotor->style.'/css/admin.css"/>
			<link rel="stylesheet" href="./'.$this->plxShow->plxMotor->aConf['racine_themes'].$this->plxShow->plxMotor->style.'/css/plucss.css"/>
			<link rel="stylesheet" href="'.PLX_PLUGINS.'Plinstagram/style.css"/>
			<link rel="favicon" type="image/png" href="'.PLX_PLUGINS.'Plinstagram/icon.png">
			<link rel="icon" type="image/png" href="'.PLX_PLUGINS.'Plinstagram/icon.png">
			<style type="text/css">body{opacity: 0;transition: opacity 1s}</style>
			</head><body onload="document.body.style.opacity=1" ><div class="container plinstagram">';
			echo '
				<h1><img src="'.PLX_PLUGINS.'Plinstagram/img/plinstagram.svg"> Publier sur mon Plinstagram </h1>
				<form action="'.plxUtils::getRacine().'admin/article.php" method="post" enctype="multipart/form-data" multiple="true" id="form">
					'.$this->getTokenPostMethod().'
					<input type="hidden" name="source" value="plinstagram"/>
					<input type="hidden" name="login" value="'. $login.'"/>
					<input type="hidden" name="author" value="'.$user.'"/>

					<input type="hidden" name="date_publication_day" value="'.date('d').'"/>
					<input type="hidden" name="date_publication_day" value="'.date('d').'"/>
					<input type="hidden" name="date_creation_day" value="'.date('d').'"/>
					<input type="hidden" name="date_update_day" value="'.date('d').'"/>

					<input type="hidden" name="date_publication_month" value="'.date('m').'"/>
					<input type="hidden" name="date_creation_month" value="'.date('m').'"/>
					<input type="hidden" name="date_update_month" value="'.date('m').'"/>

					<input type="hidden" name="date_publication_year" value="'.date('Y').'"/>
					<input type="hidden" name="date_creation_year" value="'.date('Y').'"/>
					<input type="hidden" name="date_update_year" value="'.date('Y').'"/>

					<input type="hidden" name="date_publication_time" value="'.date('H:i').'"/>
					<input type="hidden" name="date_creation_time" value="'.date('H:i').'"/>
					<input type="hidden" name="date_update_time" value="'.date('H:i').'"/>

					<input type="hidden" name="new_catname" value=""/>
					<input type="hidden" name="title_htmltag" value=""/>
					<input type="hidden" name="meta_description" value=""/>
					<input type="hidden" name="meta_keywords" value=""/>
					<input type="hidden" name="artId" value="0000"/>
					<input type="hidden" name="catId[]" value="'.$this->catId.'"/>
					<input type="hidden" name="thumbnail_title" value=""/>
					<input type="hidden" name="thumbnail_alt" value="image titre"/>
					<input type="hidden" name="url" value="'.$this->newIdArticle().'"/>
					<input type="hidden" name="chapo" value=""/>
					<input type="hidden" name="template" value="article.php"/>';
					
					

					echo '
					<div class="cadre">
					<img id="pic" src="'.PLX_PLUGINS.'Plinstagram/img/pictureplaceholder.png" onclick="document.getElementById(\'files\').click()"/>
					<section id="images"></section>
					<input id="files" type="file" name="thumbnail[]" value="" accept="image/*" multiple="true"/>
					
					
					';
					echo'
						<input id="title" type="text" name="title" value="" placeholder="Titre du billet"/>
					
					 	<textarea id="id_content" name="content" placeholder="Message d\'accompagnement"></textarea>
					</div>

					<details>
						<summary># Tags </summary>
					 	<div id="tags_list">'.$this->getTagList().'</div>
					 	<input type="text" name="tags[]" value="" list="tags_list" placeholder="nouveau tag1,nouveau tag2..."/>
					</details>
					<details>
						<summary>◐ Options </summary>
						 <select id="filter"/>
							<option value="none"  selected="selected">Aucun filtre</option>
							<option value="sepia(100%)">Sepia</option>
							<option value="saturate(0%)">Gris</option>
							<option value="saturate(0%) contrast(300%)">Noir et blanc</option>
							<option value="saturate(0%) contrast(500%)">Noir et blanc ultra</option>
							<option value="contrast(150%) brightness(150%) saturate(100%) sepia(70%)">Vintage</option>
							<option value="contrast(280%) brightness(123%) saturate(1%) sepia(100%) ">1900</option>
							<option value="hue-rotate(180deg)">Alien</option>
							<option value="invert(100%)">Invert</option>
						</select>

						<select name="allow_com"/>
							<option value="1">🏁 Commentaires autorisés</option>
							<option value="">⛔ Commentaires interdits</option>
						</select>
					 	
					</details>


					 <div class="save-fixed">
					    <button type="submit" name="draft"  ><img src="'.PLX_PLUGINS.'Plinstagram/img/save.svg"/></button>
					 	<button type="submit" name="publish"><img src="'.PLX_PLUGINS.'Plinstagram/img/plinstagram_add.svg"/> Publier</button>
					 	
					 </div></form>
				</div>
				<script>
					const MAX_SIZE = '.$this->max_size.';
					
				</script>
				<script src="'.PLX_PLUGINS.'Plinstagram/form.js"></script>
				</body></html>
			';
			exit();
		}
		
	}




	########################################
	# AdminArticlePrepend
	########################################
	# Ce hook intercepte les données postées par le formulaire du plugin 
	# avant que la page article.php ne le fasse
	# C'est là que les images sont récupérées et stockées localement, 
	# que la galerie est générée et ajoutée au corps de l'article etc.
	public function AdminArticlePrepend(){

		# on vient de poster via le bookmarklet ?
		if (!empty($_POST['source'])&&$_POST['source']=='plinstagram'){

			$_POST['tags']=trim(implode(',',$_POST['tags']),',');
			$title=empty($_POST['title'])?'No title':strip_tags($_POST['title']);
			$gallery='';

			# Si on a posté des images d'illustration, on gère l'upload
			if (!empty($_POST['data'])){
				$basename=plxUtils::title2url($title).'-';
				$foldername=$basename.time(); 
				$dest_path=$this->medias_path.'/'.$foldername.'/';
				$first='';
				if (!is_dir($dest_path)){mkdir($dest_path);}

				foreach ($_POST['data'] as $key => $file) {
					// decoding picture data
					$data=$_POST['data'][$key];
					$data=explode(',',$data);
					if (!preg_match('#data:image/(?<extension>jpg|jpeg|png|gif)#i',$data[0],$type)){continue;}
					$data=base64_decode($data[1]);

					$local_filename=$dest_path.$this->filename_base.$key.'.'.$type['extension'];

					file_put_contents($local_filename, $data);
					if ($key==0){
						# première image => en tête
						$first=$local_filename;
					}
					# images => dans une galerie
					$gallery.=$this->doGalleryItem($local_filename);
					
				}
				
			}

			# ajout de la première image comme image d'en tête
			if (!empty($first)){
				echo '<?php $_POST["thumbnail"]="'.str_replace('../../','',$first).'";?>';
			}
			# génération de la galerie si nécessaire
			if (!empty($gallery)){
				$gallery='<div class="Plinstagram-gallery">'.$gallery.'</div>';
				$_POST['content'].=$gallery;
			}

			unset($_SESSION["post"]);
			if (!isset($_POST['draft'])){$_SESSION['close_bookmarklet']=true;}
			unset($_FILES);
		}
		
	}



	##############################################################################
	# plxFeedRssArticlesXml
	##############################################################################
	# transforme le flux rss pour intégrer le permalink
	public function plxFeedRssArticlesXml(){
		echo '<?php
				$thumb="'.PLX_PLUGINS.'/Plinstagram/img/plinstagram.svg"; 
				$entries=explode("<item>",$entry);
				if (!empty($entries)){
					unset($entries[count($entries)-1]);
					$entry=implode("<item>",$entries);
				}
				if ($this->plxRecord_arts->f("thumbnail")!=""){$thumb=\'<img src="\'.$this->urlRewrite($this->plxRecord_arts->f("thumbnail")).\'" style="width:100%;height:auto"/>\';}

				if (empty($oldArtId)||$oldArtId!=$artId){
					$oldArtId=$artId;
					$entry .= "\t<item>\n";
					$entry .= "\t\t"."<title>".plxUtils::strCheck($this->plxRecord_arts->f("title"))."</title> "."\n";
					$entry .= "\t\t"."<guid>".$this->urlRewrite("?article".$artId."/".$this->plxRecord_arts->f("url"))."</guid>"."\n";
					$entry .= "\t\t"."<description><![CDATA[".$thumb.plxUtils::rel2abs($this->racine,$content)."<br/><hr/><a href=\"".$this->urlRewrite("?article".$artId."#comments")."\" title=\"commenter ou voir les commentaires sur ce lien\"><small>&#9658; Commentaires</small></a>]]></description>\n";
					$entry .= "\t\t"."<pubDate>".plxDate::dateIso2rfc822($this->plxRecord_arts->f("date"))."</pubDate>"."\n";
					$entry .= "\t\t"."<dc:creator>".plxUtils::strCheck($author)."</dc:creator>"."\n";
				}
				?>';
	}

	

	##############################################################################
	# plxAdminEditArticle
	##############################################################################
	# Récupère le fichier image s'il y en a (OBSOLETE)
	public function plxAdminEditArticle(){/*
		echo '<?php
				if (empty($content["date_update_old"])){
					$content["date_update_old"]=$content["date_creation_year"].$content["date_creation_month"].$content["date_creation_day"].substr(str_replace(":","",$content["date_creation_time"]),0,4);
				}
				?>';
		*/
	}




	########################################
	# AdminProfilTop
	########################################
	# Description: affiche le lien vers la page plinsta dans le profil
	public function AdminProfilTop(){
		$this->showBookmarkletLink();
	}




	########################################
	# Private methods
	########################################
	private function setPlxVars(){ # récupère les instances de travail
		$this->plxMotor=class_exists('plxMotor')?plxMotor::getInstance():false;
		$this->plxPlugins=$this->plxMotor->plxPlugins;
		$this->plxShow=class_exists('plxShow')?plxShow::getInstance():false;
		$this->plxAdmin=class_exists('plxAdmin')?plxAdmin::getInstance():false;
	}

	private function getTagList(){// heu...(ಠ_ಠ)
		$tags=array();
		foreach($this->plxMotor->aTags as $article){
			$tag=array_flip(array_map('trim',explode(',',$article['tags'])));
			$tags=array_merge($tags,$tag);
		}
		ksort($tags, SORT_FLAG_CASE+ SORT_NATURAL);
		
		$taglist='';
		foreach($tags as $tag=>$osef){if (!empty($tag)){$taglist.='<input type="checkbox" value="'.$tag.'" id="'.$tag.'" name="tags[]"><label for="'.$tag.'">'.$tag.'</label>';}}
 		return $taglist;
 
	 	return $tags;
	}

	private function newIdArticle(){
		$aFiles=$this->plxMotor->plxGlob_arts->aFiles;
		ksort($aFiles);
		$last=intval(array_keys($aFiles)[count($aFiles)-1])+1;
		$last=sprintf("%'.04d", $last);
		return $last;			
	}	
	private function setCatId($cats=null){
		if (!$cats&&defined('PLX_ADMIN')){
			$cats=$this->plxAdmin->aCats;
		}elseif (!$cats&&!empty($this->plxMotor)){
			$cats=$this->plxMotor->aCats;
		}else{return 'houla';}

		foreach ($cats as $id => $data) {
			if (mb_strtolower($data['name'])=='plinstagram'){
				$this->catId=$id;
				return $id;
			}
		}
	}

	########################################
	# showPinstagramRSSLink
	########################################
	# Affichage du lien vers le flux rss de la catégorie Pinstagram
	public function showPinstagramRSSLink(){
		echo '<a class="RSS_feed" href="';
		$this->plxShow->urlRewrite(PLX_ROOT.'feed.php?rss/categorie'.$this->catId);
		echo '" title="Flux RSS de '.$this->title.'">'.$this->title.'</a>';
	}
	public function catId2Name($id){
		return $this->plxMotor->aCats[$id]["name"];
	}
	public function doGalleryItem($pic){
		return '<img src="'.plxUtils::getRacine().str_replace('../../','',$pic).'" alt="'.basename($pic).'" class="Plinstagram-gallery-pic"/>';
	}
	########################################
	# showBookmarkletLink
	########################################
	# Description:
	public function showBookmarkletLink(){
		if (empty($this->plxMotor)){$this->setPlxVars();}	
		$root=$this->plxMotor->racine;
		if (defined('PLX_ADMIN')){
			$root=$this->plxAdmin->racine;
		}else{
			$root=$this->plxMotor->racine;
		}
		echo '<style>
				#profil .PlinstagramBookmarklet{
				    padding:15px;
				    background:rgba(0,0,0,0.2);
				    list-style:none;
				    margin:5px;
				    display:inline-block;
				}
				#profil .PlinstagramBookmarklet *{
				    vertical-align:middle;
				}
				#profil .PlinstagramBookmarklet img{
				    width:32px;height:auto;
				}
			</style>';
			
		echo '<li class="PlinstagramBookmarklet"><a href="'.$root.'?user='.$_SESSION['user'].'&source=plinstagram" title="Utiliser ce lien pour poster"><img src="'.PLX_PLUGINS.'Plinstagram/icon.png">'.$this->linkname.'</a></li>';
	}

	/**
	 * Méthode qui affiche le champ input contenant le token
	 *
	 * @return	stdio/null
	 * @author	Stephane F
	 **/
	# Repompe de core/lib/plxToken
	public static function getTokenPostMethod() {

		$token = sha1(mt_rand(0, 1000000));
		$_SESSION['formtoken'][$token] = time();
		return '<input name="token" value="'.$token.'" type="hidden" />';

	}


	# redirect (oui, j'ai honte  (ಥ_ಥ)   )
	###########################
	public  function redirect($url=null){
		header('location: '.$url);
		exit('<script>document.location.href="'.$url.'";</script>');
	}
}








/* Pense-bete:
 * Récuperer des parametres du fichier parameters.xml
 *	$this->getParam("<nom du parametre>")
 *	$this-> setParam ("param1", 12345, "numeric")
 *	$this->saveParams()
 *
 *	plxUtils::strCheck($string) : sanitize string
 * Accéder à une méthode de plugin depuis un hook sur une page admin
 *  echo '<?php $plxAdmin->plxPlugins->aPlugins["NOMDUPLUGIN"]->METHODE()?>';
 * 
 * Quelques constantes utiles: 
 * PLX_CORE
 * PLX_ROOT
 * PLX_CHARSET
 * PLX_PLUGINS
 * PLX_CONFIG_PATH
 * PLX_ADMIN (true si on est dans admin)
 * PLX_CHARSET
 * PLX_VERSION
 * PLX_FEED
 *
 * Appel de HOOK dans un thème
 *	eval($plxShow->callHook("ThemeEndHead","param1"))  ou eval($plxShow->callHook("ThemeEndHead",array("param1","param2")))
 *	ou $retour=$plxShow->callHook("ThemeEndHead","param1"));
 */